package com.ladwa.aditya.contact.data.local;

import android.content.Context;

import com.ladwa.aditya.contact.data.model.Contact;
import com.ladwa.aditya.contact.data.model.ContactStorIOContentResolverDeleteResolver;
import com.ladwa.aditya.contact.data.model.ContactStorIOContentResolverGetResolver;
import com.ladwa.aditya.contact.data.model.ContactStorIOContentResolverPutResolver;
import com.pushtorefresh.storio.contentresolver.ContentResolverTypeMapping;
import com.pushtorefresh.storio.contentresolver.StorIOContentResolver;
import com.pushtorefresh.storio.contentresolver.impl.DefaultStorIOContentResolver;
import com.pushtorefresh.storio.contentresolver.operations.put.PutResult;
import com.pushtorefresh.storio.contentresolver.operations.put.PutResults;
import com.pushtorefresh.storio.contentresolver.queries.Query;

import java.util.List;

import io.reactivex.Single;
import timber.log.Timber;

/**
 * Created by Aditya on 09-Feb-17.
 */

public class GoJekLocalRepository {
    private StorIOContentResolver mStorIOContentResolver;
    private Context context;

    public static GoJekLocalRepository makeGoJekLocalData(Context context) {
        return new GoJekLocalRepository(context);
    }

    private GoJekLocalRepository(Context context) {
        this.context = context;
        mStorIOContentResolver = DefaultStorIOContentResolver.builder()
                .contentResolver(context.getContentResolver())
                .addTypeMapping(Contact.class, ContentResolverTypeMapping.<Contact>builder()
                        .putResolver(new ContactStorIOContentResolverPutResolver())
                        .getResolver(new ContactStorIOContentResolverGetResolver())
                        .deleteResolver(new ContactStorIOContentResolverDeleteResolver())
                        .build()).build();
    }

    public boolean saveContactList(List<Contact> contacts) {
        PutResults<Contact> contactPutResults = mStorIOContentResolver.put().objects(contacts).prepare().executeAsBlocking();
        return contactPutResults.numberOfInserts() > 0 || contactPutResults.numberOfUpdates() > 0;
    }

    public Single<List<Contact>> getContactList() {
        List<Contact> contactList = mStorIOContentResolver.get().listOfObjects(Contact.class).withQuery(Query.builder().uri(DatabaseContract.Contacts.CONTENT_URI).build()).prepare().executeAsBlocking();
        Timber.d(String.valueOf(contactList.size()));
        return Single.just(contactList);
    }

    public boolean saveContact(Contact contact) {
        PutResult putResult = mStorIOContentResolver.put().object(contact).prepare().executeAsBlocking();
        return putResult.wasInserted() || putResult.wasUpdated();
    }
}
